@@ -1,6 +1,8 @@
 import sqlalchemy
+from sqlalchemy import inspect
 from sqlalchemy.ext.declarative import base
 from sqlalchemy.engine.reflection import Inspector
+from sqlalchemy.ext.hybrid import hybrid_property
 from sqlalchemy.orm import object_mapper, session, relationships
 
 import logic_bank
@@ -56,52 +58,89 @@ class LogicRow:
         self.engine = rb._engine
         self.some_base = declarative_base()
 
-        self.name = type(self.row).__name__
+        self.name = type(self.row).__name__  # class name (not table name)
         self.table_meta = None
         if self.row is not None:
-            self.table_meta = row.metadata.tables[type(self.row).__name__]
+            if type(self.row).__name__ in row.metadata.tables:
+                self.table_meta = row.metadata.tables[type(self.row).__name__]
+            else:
+                self.table_meta = inspect(self.row)
+                self.log_engine("Restriction: Class Name must equal Table Name: " + self.name)
         if self.engine is not None:  # e.g, for testing legacy logic (no RuleBank)
             self.inspector = Inspector.from_engine(self.engine)
 
+    def get_attr_name(self, mapper, attr)-> str:
+        """polymorhpism is for wimps - find the name
+            returns None if bad name, or is collection, or is object
+        """
+        attr_name = None
+        if hasattr(attr, "key"):
+            attr_name = attr.key
+        elif isinstance(attr, hybrid_property):
+            attr_name = attr.__name__
+        elif hasattr(attr, "__name__"):
+            attr_name = attr.__name__
+        elif hasattr(attr, "name"):
+            attr_name = attr.name
+        if attr_name == "Customerxx":
+            print("Debug Stop")
+        if hasattr(attr, "impl"):
+            if attr.impl.collection:
+                attr_name = None
+            if isinstance(attr.impl, sqlalchemy.orm.attributes.ScalarObjectAttributeImpl):
+                attr_name = None
+        return attr_name
+
     def __str__(self):
         result = ".."
         for x in range(self.nest_level):
             result += ".."
         result += self.row.__tablename__ + "["
         my_meta = self.table_meta
-        key_cols = my_meta.primary_key.columns.keys()
-        is_first = True
-        for each_key_col in key_cols:
-            if not is_first:
-                result += " | "
-            is_first = False
-            value = getattr(self.row, each_key_col)
-            if isinstance(value, str):
-                result += value
-            else:
-                result += str(value)
+        if not hasattr(my_meta, "primary_key"):
+            result += "not available"
+        else:   # my_meta.primary_key.columns.keys()
+            key_cols = my_meta.primary_key.columns.keys()
+            is_first = True
+            for each_key_col in key_cols:
+                if not is_first:
+                    result += " | "
+                is_first = False
+                value = getattr(self.row, each_key_col)
+                if isinstance(value, str):
+                    result += value
+                else:
+                    result += str(value)
         result += "]: "
         cols = self.row.__table__.columns
         sorted_cols = sorted(cols, key=lambda col: col.name)
         is_first = True
-        for each_col in sorted_cols:
-            each_col_name = each_col.name
-            if not is_first:
-                result += ", "
-            is_first = False
-            if each_col_name == "Idxx":
-                print("Debug Stop here")
-            value = getattr(self.row, each_col_name)
-            result += each_col_name + ": "
-            old_value = value
-            if self.old_row is not None:
-                old_value = getattr(self.old_row, each_col_name)
-            if value != old_value:
-                result += ' [' + str(old_value) + '-->] '
-            if isinstance(value, str):
-                result += value
+        row_mapper = object_mapper(self.row)
+        if self.row.__tablename__ == "Customerxx":
+            print("Debug Stop here")
+        for each_attr in row_mapper.all_orm_descriptors:
+            is_hybrid = isinstance(each_attr, hybrid_property)
+            each_attr_name = self.get_attr_name(mapper=row_mapper, attr=each_attr)
+            if each_attr_name is None:  # parent or child-list
+                pass   # don't print, don't even call (avoid sql)
             else:
-                result += str(value)
+                if not is_first:
+                    result += ", "
+                is_first = False
+                if each_attr_name == "Idxx":
+                    print("Debug Stop here")
+                value = getattr(self.row, each_attr_name)
+                result += each_attr_name + ": "
+                old_value = value
+                if self.old_row is not None and \
+                        hasattr(self.old_row, each_attr_name):
+                    old_value = getattr(self.old_row, each_attr_name)
+                if value != old_value:
+                    result += ' [' + str(old_value) + '-->] '
+                if isinstance(value, str):
+                    result += value
+                else:
+                    result += str(value)
         result += f'  row@: {str(hex(id(self.row)))}'
         return result  # str(my_dict)
 
@@ -133,8 +172,10 @@ class LogicRow:
             result_class = a_row.__class__
             result = result_class()
             row_mapper = object_mapper(a_row)
-            for each_attr in row_mapper.columns:  # note skips parent references
-                setattr(result, each_attr.key, getattr(a_row, each_attr.key))
+            for each_attr in row_mapper.all_orm_descriptors:
+                each_attr_name = self.get_attr_name(mapper=row_mapper, attr=each_attr)
+                if each_attr_name is not None:  # is parent or collection (parent??)
+                    setattr(result, each_attr_name, getattr(a_row, each_attr_name))
         return result
 
     def get_parent_logic_row(self, role_name: str, from_row: base = None) -> 'LogicRow':
